/*****************************************************************************/
/* RPADriver.java                                                            */
/* Driver/test program to exercise the JNI wrappers for the RPAMFUNC library */
/* Author: John Foley                                                        */
/* Date:   11/20/00                                                          */
/* Copyright 2000, QVS Software, Inc. All rights reserved.                   */
/*****************************************************************************/

import java.io.IOException;
import java.util.Vector;
import java.io.*;

import qvs.*;
import qvs.RPAMEventListener;
import qvs.RPAMEvents.*;
import qvs.RPAMDevices.*;

import qvs.RPAMApp;
import qvs.RPAMTerminal;
import qvs.RPAMDevices.POSDisplay;

/*****************************************************************************/
/* RPADriver Class                                                           */
/*****************************************************************************/
public class RPADriver implements qvs.RPAMGlobals
   {
   static int WtCnt;
   public static void ParseArgs(String[] args)
      {
                                       /* a not too sophisticated cmd line   */
                                       /* arg parser.  Options supported:    */
                                       /* -t <TermNum> and -s <StoreNum>     */
      int i;
      for (i = 0; i < args.length; i++)
         {
         if (args[i].compareTo("-t") == 0)
            {
            i++;
            TermNum = (new Integer(args[i])).intValue();
            System.out.println("Term overridden to " + TermNum);
            }
         else if (args[i].compareTo("-s") == 0)
            {
            i++;
            StoreNum = (new Integer(args[i])).intValue();
            System.out.println("Store overridden to " + StoreNum);
            }
         else if (args[i].compareTo("-a") == 0)
            {
            i++;
            TCAddr = args[i];
            System.out.println("TC address overridden to " + TCAddr);
            }
         }
      }

   public synchronized void Wakeup()
      {
      notifyAll();                     /* wake up all threads waiting on our */
                                       /* monitor                            */
      }

   public RPADriver()                  /* default constructor                */
      {                                /* create the objects we will use     */
      Handler = new EventHandler();    /* handles terminal events            */

                                       /* .. and display events              */
      ANDispHandler = new DisplayEventHandler(ANDISPLAY);
      ANDisp2Handler = new DisplayEventHandler(ANDISPLAY2);
      Serial1Handler = new SerialEventHandler(1);
      Serial2Handler = new SerialEventHandler(2);

      Term = new RPAMTerminal();       /* the Terminal object                */
      ANDisplay = Term.getANDisplay(); /* the terminal's display object      */
      App = new RPAMApp();             /* the RPAM app object for trace etc. */
      BufferedKeys = new Vector();     /* queue of keys to send to the .286  */

      rdr = new KeyboardReader();      /* A thread to read the keyboard      */

      TermNum = 1;                     /* default to terminal number 1       */
      StoreNum = 0;                    /* default to store number 0          */
      TCAddr = "";                     /* no particular TC box               */
      }
                                       /* local class to handle events fired */
                                       /* from the terminal object           */
   public class EventHandler implements qvs.RPAMEventListener
      {
      private int PrintCount = 0;

      public void IOProcWait(IOProcWaitEvent evt)
         {
         System.out.println("IOProcWaitEvent");
         }

      public void Tone(ToneEvent evt)
         {
         System.out.println("Tone Freq " + evt.Frequency +
                                                       " dur " + evt.Duration);
         }

      public void Offline(OfflineEvent evt)
         {
         System.out.println("OfflineEvent");
         };
      public void ConnectTimeout(ConnectTimeoutEvent evt)
         {
         System.out.println("ConnectTimeoutEvent");
         System.exit(1);
         };
      public void Display(DisplayEvent evt)
         {
         System.out.println("DE [" + evt.Row1Text + "][" + evt.Row2Text + "]");
         };
      public void Print(PrintEvent evt)
         {
         PrintCount = PrintCount + 1;
         if (PrintCount == 10)
            {
            Term.getPrinter().SendPrintRC(evt.Station, errPrtCoverOpen);
            PrintCount = 0;
            }
         else
            {
            System.out.println("PE [" + evt.Data + "][" + evt.LineFeeds + "][" + evt.Station + "]");
            Term.getPrinter().SendPrintRC(evt.Station, 0);
            }
         }
      public void PrintEx(PrintExEvent evt)
         {
         if (evt.Command == CMD_START_PRINT)
            {
            Term.getPrinter().SendPrintRC(evt.Station, 0);
            }

         if ((evt.Command == CMD_PRINT_DATA) && (evt.Station == CR))
            {
            System.out.println("PrintEx [" + ByteArrToString(evt.Data,0,evt.DataLen) + "]");
            }
         }
      public void ActivateMICR(ActivateMICREvent evt)
         {
         System.out.println("Activate MICR event");
         Term.getPrinter().SendMICRData("1234567890", 0);
         }
      public void Lock(LockEvent evt)
         {
         System.out.println("LockEvent");
         };
      public void Unlock(UnlockEvent evt)
         {
         System.out.println("UnlockEvent");
         };
      public void ScannerUnlock(ScannerUnlockEvent evt)
         {
         System.out.println("ScannerUnlock");
         };
      public void ScannerLock(ScannerLockEvent evt)
         {
         System.out.println("ScannerLock");
         };
      public void Online(qvs.RPAMEvents.OnlineEvent evt)
         {
         System.out.println("OnlineEvent");
         }

      public void DisplayEx(qvs.RPAMEvents.DisplayExEvent evt)
         {
                                       /* here you can use the Row1Text &    */
                                       /* Row2Text members of the evt object */
                                       /* or you can use the ANDisplay sub-  */
                                       /* object of the Term object and ask  */
                                       /* it what's on the display.  For     */
                                       /* illustrative purposes only.  You'd */
                                       /* probably always want to use the    */
                                       /* members of the evt object for      */
                                       /* efficiency's sake                  */
         // System.out.println("[" + evt.Row1Text + "][" + evt.Row2Text + "]");
         // System.out.println("[" + ANDisplay.getRow1Text() + "]["
         //                                    + ANDisplay.getRow2Text() + "]");

         Wakeup();                     /* wake up sleepers                   */

         }
      public void WaitLight(WaitLightEvent evt)
         {
         System.out.println("WaitLightEvent " + evt.NewValue);
         };

      public void MessageLight(MessageLightEvent evt)
         {
         System.out.println("MessageLightEvent " + evt.NewValue);
         };

      public void OfflineLight(OfflineLightEvent evt)
         {
         System.out.println("OfflineLightEvent " + evt.NewValue);
         };

      public void FieldLevelInput2(FieldLevelInput2Event evt)
         {
         System.out.println("FieldLevelInput2Event [" +
         evt.MaxLen + "][" + evt.EchoInput + "][" + evt.FirstDisplayCol + "][" + evt.LastDisplayCol +
         "][" + evt.StateNum + "][" +  evt.AlphaEntry + "][" +  evt.TransitionFlag + "]");
         };

      public void DrawerOpen(DrawerOpenEvent evt)
         {
         System.out.println("DrawerOpenEvent " + evt.DrawerNumber);
                                       /* when we get a drawer open event    */
                                       /* we tell the app that it's open     */
         if (evt.DrawerNumber == 1)
            {
            Term.getCashDrawer().setDrawer1Open(true);
            }
         else
            {
            Term.getCashDrawer().setDrawer2Open(true);
            }
         };

      public void FieldLevelInput(FieldLevelInputEvent evt)
         {
         System.out.println("FieldLevelInputEvent [" +
         evt.MaxLen + "][" + evt.EchoInput + "][" + evt.FirstDisplayCol + "][" + evt.LastDisplayCol +
         "][" + evt.StateNum + "]");
         };

      public void MsrLock(MsrLockEvent evt)
         {
         System.out.println("MSR lock event");
         }

      public void MsrUnlock(MsrUnlockEvent evt)
         {
         System.out.println("MSR unlock event");
         }

      public void ScaleRead(ScaleReadEvent evt)
         {
         System.out.println("Scale read event");
                                       /* send a dummy weight of 2.12 lbs    */

        /* Term.getScale().SendScaleData(212); */
         }

      public void UserData(UserDataEvent evt)
         {
         System.out.println("UData[" + ByteArrToString(evt.Data, 0, evt.Data.length) + "]");
         };
      public void RunningTotalData(RunningTotalDataEvent evt) {};
      public void TotalsWrite(TotalsWriteEvent evt) {};
      public void TotalsRead(TotalsReadEvent evt) {};
      public void TermNumberChanged(TermNumberChangedEvent evt) {};
      public void OfflineTotalsData(OfflineTotalsDataEvent evt) {};
      public void PrintEject(PrintEjectEvent evt) {};
      public void ReceiptCut(ReceiptCutEvent evt) {};
      public void TermIPLControl(TermIPLControlEvent evt) {};
      public void TermReload(TermReloadEvent evt) {};

      /* not implemented */
      public void ActivateMarqueeDisplay(ActivateMarqueeDisplayEvent evt) {};
      public void DeactivateMarqueeDisplay(DeactivateMarqueeDisplayEvent evt) {};
      public void LoadMarqueeDisplay(LoadMarqueeDisplayEvent evt) {};
      /* not implemented */


      }

                                       /* local class to handle events fired */
                                       /* from the ANDisplay object          */
   public class DisplayEventHandler implements qvs.DisplayEventListener
      {
      int Device;

      public DisplayEventHandler(int DeviceID)
         {
         Device = DeviceID;
         }

      public void DisplayEx(DisplayExEvent evt)
         {
         System.out.println("DisplayEx [" + Device + "][" + evt.UpdateFlags +
                              "][" + evt.Row1Text + "][" + evt.Row2Text + "]");
         }

      public void Display(DisplayEvent evt)
         {
         System.out.println("Display [" + Device + "][" +
                                     evt.Row1Text + "][" + evt.Row2Text + "]");
         }
      }

                                       /* local class to handle events fired */
                                       /* from the Serial? objects           */
   public class SerialEventHandler implements qvs.SerialEventListener
      {
      int Port;

      public SerialEventHandler(int pPort)
         {
         Port = pPort;
         }

      public void Open(SerialOpenEvent evt)
         {
         int GetLongVal;
         System.out.println("Serial open for port " + Port);
         System.out.println("Baud " + evt.Speed + " Parity " + evt.Parity);
         System.out.println("DBits " + evt.DataBits + " SBits " + evt.StopBits);
         System.out.println("Buffer size " + evt.BuffSize);

                                       /* send a good open return code       */
                                       /* and send a getlong status update   */
                                       /* to TC.  Note: the only bits that   */
                                       /* may be set in a getlong from the   */
                                       /* client are the ones shown here     */
         GetLongVal = s1DataLost | s2DSRActive | s2CTSActive | s2ParityError |
                                             s2FramingError | s2BreakDetected;

         GetLongVal = s2CTSActive | s2DSRActive;

         if (Port == 1)
            {
            Term.getSerial1().SendOpenRc(0);
            Term.getSerial1().UpdatePortStatus(GetLongVal);
            }
         else if (Port == 2)
            {
            Term.getSerial2().SendOpenRc(0);
            Term.getSerial2().UpdatePortStatus(GetLongVal);
            }
         }

      public void PutLong(SerialPutLongEvent evt)
         {
         System.out.println( "PutLong on port " + Port + " Timeout " +
                                  evt.TimeOut + " Stat " + evt.PortStatusInfo);

                                       /* send a good  return code back to   */
                                       /* the .286                           */
         POSSerialDevice dev = (POSSerialDevice)(evt.getSource());
         dev.SendPutLongRc(0);
         }

      public void Write(SerialWriteEvent evt)
         {
         System.out.println("Serial write for " + evt.Length + " bytes.");
         String StrData = ByteArrToString(evt.Data,0,evt.Length);
         System.out.println("[" + StrData + "]" );

                                       /* give an rc back for the number of  */
                                       /* bytes written                      */
         POSSerialDevice dev = (POSSerialDevice)(evt.getSource());
         dev.SendWriteRc(evt.Length);

         }

      public void Close(SerialCloseEvent evt)
         {
         System.out.println("Close for port " + Port);
         }
      }
                                       /* local class to run a thread to     */
                                       /* read the keyboard                  */
   public class KeyboardReader extends Thread
      {
      char[] InKeys = new char[100];
      int CharsRead = 0;
      int x;
      InputStreamReader isr = new InputStreamReader(System.in);

      char[] KeyMap =                  /* table to translate ASCII key codes */
                                       /* to SA function codes               */
         {

         /*
         Ascii Key      SA FunctionCode
         */

         '/',           (char)0x4e,    /* /               */
         '0',           (char)0x30,    /* 0               */
         '1',           (char)0x31,    /* 1               */
         '2',           (char)0x32,    /* 2               */
         '3',           (char)0x33,    /* 3               */
         '4',           (char)0x34,    /* 4               */
         '5',           (char)0x35,    /* 5               */
         '6',           (char)0x36,    /* 6               */
         '7',           (char)0x37,    /* 7               */
         '8',           (char)0x38,    /* 8               */
         '9',           (char)0x39,    /* 9               */
         's',           (char)0x3d,    /* Signon  S       */
         'c',           (char)0x49,    /* Clear   E       */
         'e',           (char)0x50,    /* Enter   R       */
         't',           (char)0x51,    /* Total   T       */
         'h',           (char)0x5b,    /* Cash    H       */
         'w',           (char)0x48,    /* Weight          */
         (char)0x00,    (char)0x00     /* End of data     */
         };

//char[] KeyMap =                  /* table to translate ASCII key codes */
//                                     /* to GSA function codes              */
//       {
//
//       /*
//       Ascii Key      GSA FunctionCode
//       */
//
//       '/',           78,    /* /               */
//       '0',           48,    /* 0               */
//       '1',           49,    /* 1               */
//       '2',           50,    /* 2               */
//       '3',           51,    /* 3               */
//       '4',           52,    /* 4               */
//       '5',           53,    /* 5               */
//       '6',           54,    /* 6               */
//       '7',           55,    /* 7               */
//       '8',           56,    /* 8               */
//       '9',           57,    /* 9               */
//       's',           76,    /* Signon          */
//       'c',           73,    /* Clear           */
//       'e',           95,    /* Enter           */
//       't',           75,    /* Total           */
//       'h',           69,    /* Cash            */
//       'k',          102,    /* Credit          */
//       (char)0x00,    (char)0x00     /* End of data     */
//       };

                                       /* routine to translate keystrokes to */
                                       /* function codes                     */

      public int GetFCodeFromKey(int InKey)
         {
         int RetFCode = 0;
         int idx;
         for(idx=0;(KeyMap[idx] != 0);idx+=2)
            {
            if (KeyMap[idx] == InKey)
               {
               RetFCode = KeyMap[idx+1];
               break;
               }
            }
         return RetFCode;
         }

      public void run()
         {
         int FCode;
         try
            {
            for(;;)                    /* loop forever                       */
               {                       /* block waiting for some keystrokes  */
               CharsRead = isr.read(InKeys,0,1);
               for (x=0;x<CharsRead;x++)
                  {
                  if (InKeys[x] == '&')
                     {                 /* scanner test case                  */
                     Term.getScanner().SendScannerData(10, "04963406");
                     }
                  else if (InKeys[x] == 'o')
                                       /* msr test case                      */
                     {
                     System.out.println("Sending MSR data");

                     Term.getMsr().SendMsrData(
                                    "B4387751111110^T/^99121010123456789", 35,
                                           "4387751111110=991210", 20,"", -1);
                     }
                  else if (InKeys[x] == 'z')
                                       /* scale test case                    */
                     {
                     Term.getScale().SendScaleData(0x80a30563);
                     }
                  else if (InKeys[x] == 'p')
                                       /* scale test case                    */
                     {
                     Term.getScale().SendScaleData(212);
                     }
                  else if (InKeys[x] == '@')
                                       /* serial test case                  */
                     {
                     String TestStr = new String("12345__________54321");
                     Term.getSerial1().Write(20, TestStr.getBytes());
                     TestStr = "00000----------00000";
                     Term.getSerial2().Write(20, TestStr.getBytes());

                     }
                  else if (InKeys[x] == '%')
                                       /* update drawer 1 status to closed   */
                     {
                     Term.getCashDrawer().setDrawer1Open(false);
                     }
                  else if (InKeys[x] == '^')
                                       /* update drawer 2 status to closed   */
                     {
                     Term.getCashDrawer().setDrawer2Open(false);
                     }
                  else if (InKeys[x] == '+')
                                       /* do some file operations            */
                     {
                     FileOps();
                     }
                  else if (InKeys[x] == '!')
                     {
                     /* Exit test program */
                     System.out.println("Exiting...");
                     System.exit(0);
                     }
                  else
                     {
                     FCode = GetFCodeFromKey((int)InKeys[x]);
                     if (FCode > 0)
                        {
                        Term.getKeyboard().SendKeyboardData(FCode);
                        }
                     }
                  }
               }
            }
         catch(IOException ex)
            {
            System.err.println("IOException: " +  ex.getMessage());
            }
         }
      }

   EventHandler Handler;               /* an instance of the event handler   */
                                       /* interface                          */
   DisplayEventHandler ANDispHandler;  /* an instance of the DisplayEvent    */
                                       /* hander for ANDisplay               */
   DisplayEventHandler ANDisp2Handler; /* an instance of the DisplayEvent    */
                                       /* hander for ANDisplay2              */
   SerialEventHandler Serial1Handler;  /* an instance of the SerialEvent     */
                                       /* hander for Serial1                 */
   SerialEventHandler Serial2Handler;  /* an instance of the SerialEvent     */
                                       /* hander for Serial2                 */


   RPAMTerminal Term;                  /* a Terminal object to use           */
   RPAMApp App;                        /* an App objec to use                */
   POSDisplay ANDisplay;               /* a POSDisplay to use                */
   Vector BufferedKeys;                /* queued keys to send                */
   KeyboardReader rdr;                 /* input reader thread                */
   static int TermNum;                 /* terminal number we'll connect as   */
   static int StoreNum;                /* our store number                   */
   static String TCAddr;               /* IP addr of TC box to talk to       */

                                       /* app main line code                 */
   public static void main(String args[])
      {
      char InChar;
      int Avail;

      System.out.println("JVM version " + System.getProperty("java.version"));

      RPADriver Dvr = new RPADriver(); /* get an instance of the app class   */
                                       /* created                            */

      Dvr.ParseArgs(args);             /* get our cmd line arguments         */

                                       /* add our event handler to the list  */
                                       /* of listener for RPAM events from   */
                                       /* the terminal object                */
      Dvr.Term.addRPAMEventListener(Dvr.Handler);

                                       /* also add ourselves to each of the  */
                                       /* ANDisplay objects' listener list   */
                                       /* for DisplayEvents                  */
      Dvr.Term.getANDisplay().addDisplayEventListener(Dvr.ANDispHandler);
      Dvr.Term.getANDisplay2().addDisplayEventListener(Dvr.ANDisp2Handler);

                                       /* also add ourselves to each of the  */
                                       /* serial objects' listener list      */
                                       /* for SerialEvents                   */
      Dvr.Term.getSerial1().addSerialEventListener(Dvr.Serial1Handler);
      Dvr.Term.getSerial2().addSerialEventListener(Dvr.Serial2Handler);

      Dvr.App.setTracing(1);           /* turn RPAM tracing on               */

      Dvr.App.setOptimizeDisplay(0);   /* tell RPAM we want to see ALL of    */
                                       /* the display messages               */

      Dvr.App.setTermNumAssignMode(1); /* we're going to assing term nums    */


      if (TCAddr.compareTo("") != 0)
         {
         System.out.println("Setting TC address to " + TCAddr);
         Dvr.App.setTCServerAddresses(TCAddr, "0.0.0.0");
         }

      Dvr.Term.setUnitID(1);           /* set our unit id                    */

      Dvr.Term.setTermNum(TermNum);    /* set our terminal number            */

                                       /* tell the terminal we want to be    */
                                       /* advised about field level input    */
                                       /* cmds.                              */
      Dvr.Term.getIOProc().putFieldLevelInputMode(FL_RP_STATES);

      if (StoreNum != 0)
         {
         Dvr.App.setStoreNum(StoreNum);/* TESTING purposes only.  Should     */
                                       /* never set StoreNum. Note: for now  */
                                       /* you must do this after you set     */
                                       /* unit id for the terminal.          */
         }


                                       /* tell the RPAM library that we want */
                                       /* to handle cash drawer processing   */
      Dvr.Term.getCashDrawer().setDeviceResponseMode(rpaAppDeviceResponse);

                                       /* tell the RPAM layer that we have   */
                                       /* both cash drawers and that both    */
                                       /* are closed                         */
      Dvr.Term.getCashDrawer().setDrawer1Present(true);
      Dvr.Term.getCashDrawer().setDrawer2Present(true);


      Dvr.Term.getCashDrawer().setDrawer1Open(false);
      Dvr.Term.getCashDrawer().setDrawer2Open(false);

                                       /* tell the .286 we have a mod 3/4    */
                                       /* printer attached                   */
      Dvr.Term.getPrinter().setDeviceID(rpaMod34prt);
      Dvr.Term.getPrinter().setDeviceResponseMode(rpaAppDeviceResponse);

                                       /* and for testing say no MICR attach
      Dvr.Term.getPrinter().putMICRAttached(false);                          */


                                       /* enable the serial ports            */
                                       /* uncomment out the following 2 lines*/
                                       /* to enable serial ports             */
//    Dvr.Term.getSerial1().setDeviceEnabled();
//    Dvr.Term.getSerial2().setDeviceEnabled();


      System.out.println("Attempting Connect to TC...");

      Dvr.Term.Connect();              /* get the TC session rolling...      */

      Dvr.rdr.start();                 /* get the keyboard thread started    */

      for (;;)                         /* loop forever                       */
         {
         synchronized(Dvr)
            {
            try
               {
               Dvr.wait();             /* wait for something to happen       */

                                       /* dump the 'state' of the terminal   */
               System.out.println("[" + Dvr.ANDisplay.getRow1Text() + "][" +
                                             Dvr.ANDisplay.getRow2Text()+ "]");
               }
            catch(InterruptedException ex)
               {
               System.err.println("InterruptedException: " +  ex.getMessage());
               }
            }
         }
      }

   public String ByteArrToString(byte[] arr, int start, int len)
      {
      String RetVal = "";
      int i;
      for (i=start;i<start+len;i++)
         {
         RetVal = RetVal + (char)arr[i];
         }
      return RetVal;
      }

   public byte[] StringToByteArr(String str)
      {
      int Len = str.length();
      byte[] RetVal = new byte[Len];
      int i;
      for (i=0;i<Len;i++)
         {
         RetVal[i] = (byte)(str.charAt(i));
         }
      return RetVal;

      }

   public void FileOps()
      {
      POSFile PFile;
      POSKeyedFile PKFile;
      PFile = new POSFile();
      PKFile = new POSKeyedFile();
      int x;
      int y;
      int BytesRead=0;
      int CumBytesRead;
      int Offset;
      byte[] buf = new byte[25];
      int rc;


      PFile.setUnitID(1);
      PFile.setName("R::ADX_UDT1\\FILEOP.DAT");

      // create the file with read/write/delete permissions
      PFile.Create(rpaFlgDelete + rpaFlgWrite + rpaFlgRead   , 0x400, 0, 0);

      if (PFile.getErrorCode4690() == 0)
         {
         // now write some data to the file

         for (x=0;x<20;x++)
            {
            for (y=0;y<25;y++)
              buf[y] = (byte)(x + 0x30);

            PFile.Write(rpaSeekFP, buf, 25, 0);

            }
            // close the file
            PFile.Close();
         }
      else
         {
         System.out.println("Create of FILEOP.DAT failed " +  PFile.getErrorCode4690());
         }

         // now read in a couple of buffers from the file
         PFile.Open(rpaFlgRead, 0);
         if (PFile.getErrorCode4690() == 0)
            {
            // read 25 bytes from offset 35
            BytesRead = PFile.Read(rpaSeekBOF, buf, 25, 35);

            // read 12 bytes from offset 47
            BytesRead = PFile.Read(rpaSeekBOF, buf, 12, 47);

            // now read sequentially through the file
            BytesRead = 1;
            CumBytesRead = 0;
            Offset = 0;
            while (BytesRead > 0)
               {
               BytesRead = PFile.Read(rpaSeekBOF, buf, 24, Offset);
               if (BytesRead == 0)
                  {
                  System.out.println("EOF found, cumulative bytes read " + CumBytesRead);
                  System.out.println("ErrCode  " + PFile.getErrorCode4690());
                  }
               else if (BytesRead <0)
                  {
                  System.out.println("4690 error " + PFile.getErrorCode4690());
                  }
               else
                  {
                  CumBytesRead = CumBytesRead + BytesRead;
                  Offset = Offset + BytesRead;
                  }
               }
            // close the file
            PFile.Close();
            }
         else
            {
            System.out.println("Error on FILEOP.DAT open " +
                                                    PFile.getErrorCode4690());
            }

      // now do some keyed file ops
      // set the unit ID and name
      // unit ID must match unit ID of the terminal you want to
      // use
      PKFile.setUnitID(1);
      PKFile.setName("R::ADX_UDT1\\KEYED.FIL");

      // open the file with read/write permissions
      PKFile.Open(rpaFlgWrite + rpaFlgRead , 0);
      if (PKFile.getErrorCode4690() == 0)
         {
         // write a record with key value 'KEY02' to the file
         // the sample keyed file 'KEYED.FIL' has record length
         // of 20 with a key length of 5
         buf = StringToByteArr("KEY02X_____________Y");
         rc = PKFile.Write(0, buf, 20);
         buf = StringToByteArr("KEY98XjjfjjfjjfjjfjY");
         rc = PKFile.Write(0, buf, 20);
         buf = StringToByteArr("KEY99X9_9_9_9_9_9_9Y");
         rc = PKFile.Write(0, buf, 20);
         buf = StringToByteArr("KEY9AX9a9a9a9a9a9a9Y");
         rc = PKFile.Write(0, buf, 20);

         // put the key value to read in the buffer
         buf = StringToByteArr("                    ");
         buf = StringToByteArr("KEY02               ");

         // read the data
         rc = PKFile.Read(0, buf, 20, 5);

         // delete some records
         buf = StringToByteArr("KEY01");
         rc = PKFile.DeleteRecord(buf, 5);
         buf = StringToByteArr("KEY98");
         rc = PKFile.DeleteRecord(buf, 5);

         // close the file
         PKFile.Close(0);
         }
      else
         {
         System.out.println("Error on KEYED.FIL open " +
                                                     PFile.getErrorCode4690());
         }
      }
   }

